pub fn template<F, O>(func: F) -> FnTemplate<F, O>Expand description
Returns a “free floating” template for a given func. This prevents the need to define a custom type for one-off templates.
pub fn template<F, O>(func: F) -> FnTemplate<F, O>Returns a “free floating” template for a given func. This prevents the need to define a custom type for one-off templates.