FilteredResourcesMut

Struct FilteredResourcesMut 

pub struct FilteredResourcesMut<'w, 's> { /* private fields */ }
๐Ÿ‘ŽDeprecated since 0.20.0:

Use QueryState and QueryBuilder instead.

Provides mutable access to a set of Resources defined by the contained Access.

Use FilteredResources if you only need read-only access to resources.

To be useful as a SystemParam, this must be configured using a FilteredResourcesMutParamBuilder to build the system using a SystemParamBuilder.

Examples

// Use `FilteredResourcesMutParamBuilder` to declare access to resources.
let system = (FilteredResourcesMutParamBuilder::new(|builder| {
    builder.add_write::<B>().add_read::<C>().add_write::<D>();
}),)
    .build_state(&mut world)
    .build_system(resource_system);

world.init_resource::<A>();
world.init_resource::<C>();
world.init_resource::<D>();

fn resource_system(mut res: FilteredResourcesMut) {
    // The resource exists, but we have no access, so we can't read it or write it.
    assert!(res.get::<A>().is_err());
    assert!(res.get_mut::<A>().is_err());
    // The resource doesn't exist, so we can't read it or write it.
    assert!(res.get::<B>().is_err());
    assert!(res.get_mut::<B>().is_err());
    // The resource exists and we have read access, so we can read it but not write it.
    let c = res.get::<C>().unwrap();
    assert!(res.get_mut::<C>().is_err());
    // The resource exists and we have write access, so we can read it or write it.
    let d = res.get::<D>().unwrap();
    let d = res.get_mut::<D>().unwrap();
    // The type parameter can be left out if it can be determined from use.
    let c: Ref<C> = res.get().unwrap();
}

This can be used alongside ordinary Res and ResMut parameters if they do not conflict.

let system = (
    FilteredResourcesMutParamBuilder::new(|builder| {
        builder.add_read::<A>().add_write::<B>();
    }),
    ParamBuilder,
    ParamBuilder,
)
    .build_state(&mut world)
    .build_system(resource_system);

// Read access to A does not conflict with read access to A or write access to C.
// Write access to B does not conflict with access to A or C.
fn resource_system(mut filtered: FilteredResourcesMut, res_a: Res<A>, res_mut_c: ResMut<C>) {
    let res_a_2: Ref<A> = filtered.get::<A>().unwrap();
    let res_mut_b: Mut<B> = filtered.get_mut::<B>().unwrap();
}

But it will conflict if it tries to read the same resource that another parameter writes, or write the same resource that another parameter reads.

โ“˜
let system = (
    FilteredResourcesMutParamBuilder::new(|builder| {
        builder.add_write::<A>();
    }),
    ParamBuilder,
)
    .build_state(&mut world)
    .build_system(invalid_resource_system);

// Read access to A conflicts with write access to A.
fn invalid_resource_system(filtered: FilteredResourcesMut, res_a: Res<A>) { }

Implementations

Source

impl<'w, 's> FilteredResourcesMut<'w, 's>

pub fn as_readonly(&self) -> FilteredResources<'_, 's>

Gets read-only access to all of the resources this FilteredResourcesMut can access.

pub fn reborrow(&mut self) -> FilteredResourcesMut<'_, 's>

Returns a new instance with a shorter lifetime. This is useful if you have &mut FilteredResourcesMut, but you need FilteredResourcesMut.

pub fn access(&self) -> &Access

Returns a reference to the underlying Access.

pub fn has_read<R>(&self) -> bool
where R: Resource,

Returns true if the FilteredResources has read access to the given resource. Note that Self::get() may still return Err if the resource does not exist.

pub fn has_write<R>(&self) -> bool
where R: Resource,

Returns true if the FilteredResources has write access to the given resource. Note that Self::get_mut() may still return Err if the resource does not exist.

pub fn get<R>(&self) -> Result<Ref<'_, R>, ResourceFetchError>
where R: Resource,

Gets a reference to the resource of the given type if it exists and the FilteredResources has access to it.

pub fn get_by_id( &self, component_id: ComponentId, ) -> Result<Ptr<'_>, ResourceFetchError>

Gets a pointer to the resource with the given ComponentId if it exists and the FilteredResources has access to it.

pub fn get_mut<R>(&mut self) -> Result<Mut<'_, R>, ResourceFetchError>
where R: Resource,

Gets a mutable reference to the resource of the given type if it exists and the FilteredResources has access to it.

pub fn get_mut_by_id( &mut self, component_id: ComponentId, ) -> Result<MutUntyped<'_>, ResourceFetchError>

Gets a mutable pointer to the resource with the given ComponentId if it exists and the FilteredResources has access to it.

pub fn into_mut<R>(self) -> Result<Mut<'w, R>, ResourceFetchError>
where R: Resource,

Consumes self and gets mutable access to resource of the given type with the world 'w lifetime if it exists and the FilteredResources has access to it.

pub fn into_mut_by_id( self, component_id: ComponentId, ) -> Result<MutUntyped<'w>, ResourceFetchError>

Consumes self and gets mutable access to resource with the given ComponentId with the world 'w lifetime if it exists and the FilteredResources has access to it.