Mat4

Struct Mat4 

#[repr(C)]
pub struct Mat4 { pub x_axis: Vec4, pub y_axis: Vec4, pub z_axis: Vec4, pub w_axis: Vec4, }
A 4x4 column major matrix.

If you are primarily dealing with 3D affine transformations considering using Affine3A which is faster than a 4x4 matrix for some affine operations.

Affine transformations including 3D translation, rotation and scale can be created using methods such as Self::from_translation(), Self::from_quat(), Self::from_scale() and Self::from_scale_rotation_translation().

The Self::transform_point3() and Self::transform_vector3() convenience methods are provided for performing affine transformations on 3D vectors and points. These multiply 3D inputs as 4D vectors with an implicit w value of 1 for points and 0 for vectors respectively. These methods assume that Self contains a valid affine transform.

SIMD vector types are used for storage on supported platforms.

This type is 16 byte aligned.

Fields

x_axis: Vec4y_axis: Vec4z_axis: Vec4w_axis: Vec4

Implementations

Source

impl Mat4

pub const ZERO: Mat4

A 4x4 matrix with all elements set to 0.0.

pub const IDENTITY: Mat4

A 4x4 identity matrix, where all diagonal elements are 1, and all off-diagonal elements are 0.

pub const NAN: Mat4

All NAN:s.

pub const fn from_cols( x_axis: Vec4, y_axis: Vec4, z_axis: Vec4, w_axis: Vec4, ) -> Mat4

Creates a 4x4 matrix from four column vectors.

See also Self::from_rows when the data is in row major order.

pub const fn from_rows(row0: Vec4, row1: Vec4, row2: Vec4, row3: Vec4) -> Mat4

Creates a 4x4 matrix from four row vectors.

Matrices are stored in column major order, so the given rows are permuted into the matrix layout. Use Self::from_cols instead when the data is already in column major order.

pub const fn from_cols_array(m: &[f32; 16]) -> Mat4

Creates a 4x4 matrix from a [f32; 16] array stored in column major order.

If the data is in row major order use Self::from_rows_array instead.

pub const fn to_cols_array(&self) -> [f32; 16]

Creates a [f32; 16] array storing data in column major order.

If you require the data in row major order use Self::to_rows_array instead.

pub const fn from_cols_array_2d(m: &[[f32; 4]; 4]) -> Mat4

Creates a 4x4 matrix from a [[f32; 4]; 4] 4D array stored in column major order.

If the data is in row major order transpose the returned matrix.

pub const fn to_cols_array_2d(&self) -> [[f32; 4]; 4]

Creates a [[f32; 4]; 4] 4D array storing data in column major order.

If you require row major order transpose the matrix first.

pub const fn from_rows_array(m: &[f32; 16]) -> Mat4

Creates a 4x4 matrix from a [f32; 16] array stored in row major order.

Matrices are stored in column major order, so the array is permuted into the matrix layout. Use Self::from_cols_array instead when the data is already in column major order.

pub const fn to_rows_array(&self) -> [f32; 16]

Creates a [f32; 16] array storing data in row major order.

Matrices are stored in column major order, so the array is permuted out of the column major storage. Use Self::to_cols_array instead when you want data in column major order.

pub const fn from_diagonal(diagonal: Vec4) -> Mat4

Creates a 4x4 matrix with its diagonal set to diagonal and all other entries set to 0.

pub fn from_scale_rotation_translation( scale: Vec3, rotation: Quat, translation: Vec3, ) -> Mat4

Creates an affine transformation matrix from the given 3D scale, rotation and translation.

The resulting matrix can be used to transform 3D points and vectors. See Self::transform_point3() and Self::transform_vector3().

Panics

Will panic if rotation is not normalized when glam_assert is enabled.

pub fn from_rotation_translation(rotation: Quat, translation: Vec3) -> Mat4

Creates an affine transformation matrix from the given 3D translation.

The resulting matrix can be used to transform 3D points and vectors. See Self::transform_point3() and Self::transform_vector3().

Panics

Will panic if rotation is not normalized when glam_assert is enabled.

pub fn to_scale_rotation_translation(&self) -> (Vec3, Quat, Vec3)

Extracts scale, rotation and translation from self. The input matrix is expected to be a 3D affine transformation matrix otherwise the output will be invalid.

Panics

Will panic if self is not a valid affine transformation matrix, if the determinant of the 3x3 linear part (the rotation and scale part of the transform) is zero, when glam_assert is enabled.

pub fn from_quat(rotation: Quat) -> Mat4

Creates an affine transformation matrix from the given rotation quaternion.

The resulting matrix can be used to transform 3D points and vectors. See Self::transform_point3() and Self::transform_vector3().

Panics

Will panic if rotation is not normalized when glam_assert is enabled.

pub fn from_mat3(m: Mat3) -> Mat4

Creates an affine transformation matrix from the given 3x3 linear transformation matrix.

The resulting matrix can be used to transform 3D points and vectors. See Self::transform_point3() and Self::transform_vector3().

pub fn from_mat3_translation(mat3: Mat3, translation: Vec3) -> Mat4

Creates an affine transformation matrics from a 3x3 matrix (expressing scale, shear and rotation) and a translation vector.

Equivalent to Mat4::from_translation(translation) * Mat4::from_mat3(mat3)

pub fn from_mat3a(m: Mat3A) -> Mat4

Creates an affine transformation matrix from the given 3x3 linear transformation matrix.

The resulting matrix can be used to transform 3D points and vectors. See Self::transform_point3() and Self::transform_vector3().

pub fn from_translation(translation: Vec3) -> Mat4

Creates an affine transformation matrix from the given 3D translation.

The resulting matrix can be used to transform 3D points and vectors. See Self::transform_point3() and Self::transform_vector3().

pub fn from_axis_angle(axis: Vec3, angle: f32) -> Mat4

Creates an affine transformation matrix containing a 3D rotation around a normalized rotation axis of angle (in radians).

The resulting matrix can be used to transform 3D points and vectors. See Self::transform_point3() and Self::transform_vector3().

Panics

Will panic if axis is not normalized when glam_assert is enabled.

pub fn from_euler(order: EulerRot, a: f32, b: f32, c: f32) -> Mat4

Creates a affine transformation matrix containing a rotation from the given euler rotation sequence and angles (in radians).

The resulting matrix can be used to transform 3D points and vectors. See Self::transform_point3() and Self::transform_vector3().

pub fn to_euler(&self, order: EulerRot) -> (f32, f32, f32)

Extract Euler angles with the given Euler rotation order.

Note if the upper 3x3 matrix contain scales, shears, or other non-rotation transformations then the resulting Euler angles will be ill-defined.

Panics

Will panic if any column of the upper 3x3 rotation matrix is not normalized when glam_assert is enabled.

pub fn from_rotation_x(angle: f32) -> Mat4

Creates an affine transformation matrix containing a 3D rotation around the x axis of angle (in radians).

The resulting matrix can be used to transform 3D points and vectors. See Self::transform_point3() and Self::transform_vector3().

pub fn from_rotation_y(angle: f32) -> Mat4

Creates an affine transformation matrix containing a 3D rotation around the y axis of angle (in radians).

The resulting matrix can be used to transform 3D points and vectors. See Self::transform_point3() and Self::transform_vector3().

pub fn from_rotation_z(angle: f32) -> Mat4

Creates an affine transformation matrix containing a 3D rotation around the z axis of angle (in radians).

The resulting matrix can be used to transform 3D points and vectors. See Self::transform_point3() and Self::transform_vector3().

pub fn from_scale(scale: Vec3) -> Mat4

Creates an affine transformation matrix containing the given 3D non-uniform scale.

The resulting matrix can be used to transform 3D points and vectors. See Self::transform_point3() and Self::transform_vector3().

Panics

Will panic if all elements of scale are zero when glam_assert is enabled.

pub const fn from_cols_slice(slice: &[f32]) -> Mat4

Creates a 4x4 matrix from the first 16 values in slice.

See also Self::from_rows_slice when the slice is in row major order.

Panics

Panics if slice is less than 16 elements long.

pub fn write_cols_to_slice(&self, slice: &mut [f32])

Writes the columns of self to the first 16 elements in slice.

Panics

Panics if slice is less than 16 elements long.

pub const fn from_rows_slice(slice: &[f32]) -> Mat4

Creates a 4x4 matrix from the first 16 values in slice, stored in row major order.

Matrices are stored in column major order, so the slice is permuted into the matrix layout. Use Self::from_cols_slice instead when the slice is already in column major order.

Panics

Panics if slice is less than 16 elements long.

pub fn col(&self, index: usize) -> Vec4

Returns the matrix column for the given index.

Panics

Panics if index is greater than 3.

pub fn col_mut(&mut self, index: usize) -> &mut Vec4

Returns a mutable reference to the matrix column for the given index.

Panics

Panics if index is greater than 3.

pub fn row(&self, index: usize) -> Vec4

Returns the matrix row for the given index.

See also Self::set_row when you need to change the row.

Panics

Panics if index is greater than 3.

pub fn set_row(&mut self, index: usize, row: Vec4)

Sets the matrix row for the given index.

Matrices are stored in column major order, so the row is spread across all 4 columns and writing it touches every column. Use Self::col_mut instead when you can work with columns. See also Self::row.

Panics

Panics if index is greater than 3.

pub fn is_finite(&self) -> bool

Returns true if, and only if, all elements are finite. If any element is either NaN, positive or negative infinity, this will return false.

pub fn is_nan(&self) -> bool

Returns true if any elements are NaN.

pub fn transpose(&self) -> Mat4

Returns the transpose of self.

pub fn diagonal(&self) -> Vec4

Returns the diagonal of self.

pub fn determinant(&self) -> f32

Returns the determinant of self.

pub fn inverse(&self) -> Mat4

Returns the inverse of self.

If the matrix is not invertible the returned matrix will be invalid. The returned matrix will also be invalid if the inverse is not finite, which can happen when self contains very large or very small values. Use Self::try_inverse or Self::inverse_or_zero to detect these cases.

Panics

Will panic if the resulting inverted matrix is not finite when glam_assert is enabled.

pub fn try_inverse(&self) -> Option<Mat4>

Returns the inverse of self or None if the matrix is not invertible, or if the inverse is not finite.

pub fn inverse_or_zero(&self) -> Mat4

Returns the inverse of self or Mat4::ZERO if the matrix is not invertible, or if the inverse is not finite.

pub fn look_to_lh(eye: Vec3, dir: Vec3, up: Vec3) -> Mat4

👎Deprecated since 0.33.1:

use the glam::camera::lh::view::look_to_mat4 function instead

Creates a left-handed view matrix using a camera position, a facing direction and an up direction

For a view coordinate system with +X=right, +Y=up and +Z=forward.

Panics

Will panic if dir or up are not normalized, or if dir and up are parallel, when glam_assert is enabled.

pub fn look_to_rh(eye: Vec3, dir: Vec3, up: Vec3) -> Mat4

👎Deprecated since 0.33.1:

use the glam::camera::rh::view::look_to_mat4 function instead

Creates a right-handed view matrix using a camera position, a facing direction, and an up direction.

For a view coordinate system with +X=right, +Y=up and +Z=back.

Panics

Will panic if dir or up are not normalized, or if dir and up are parallel, when glam_assert is enabled.

pub fn look_at_lh(eye: Vec3, center: Vec3, up: Vec3) -> Mat4

👎Deprecated since 0.33.1:

use the glam::camera::lh::view::look_at_mat4 function instead

Creates a left-handed view matrix using a camera position, a focal points and an up direction.

For a view coordinate system with +X=right, +Y=up and +Z=forward.

Panics

Will panic if up is not normalized, if center is equal to eye, or if the view direction is parallel to up, when glam_assert is enabled.

pub fn look_at_rh(eye: Vec3, center: Vec3, up: Vec3) -> Mat4

👎Deprecated since 0.33.1:

use the glam::camera::rh::view::look_at_mat4 function instead

Creates a right-handed view matrix using a camera position, a focal point, and an up direction.

For a view coordinate system with +X=right, +Y=up and +Z=back.

Panics

Will panic if up is not normalized, if center is equal to eye, or if the view direction is parallel to up, when glam_assert is enabled.

pub fn frustum_rh_gl( left: f32, right: f32, bottom: f32, top: f32, z_near: f32, z_far: f32, ) -> Mat4

👎Deprecated since 0.33.1:

use the glam::camera::rh::proj::opengl::frustum function instead

Creates a right-handed perspective projection matrix with [-1,1] depth range.

This is the same as the OpenGL glFrustum function.

See https://registry.khronos.org/OpenGL-Refpages/gl2.1/xhtml/glFrustum.xml

pub fn frustum_lh( left: f32, right: f32, bottom: f32, top: f32, z_near: f32, z_far: f32, ) -> Mat4

👎Deprecated since 0.33.1:

use the glam::camera::lh::proj::directx::frustum function instead

Creates a left-handed perspective projection matrix with [0,1] depth range.

Panics

Will panic if left equals right, bottom equals top, z_near equals z_far, or z_near or z_far are not positive when glam_assert is enabled.

pub fn frustum_rh( left: f32, right: f32, bottom: f32, top: f32, z_near: f32, z_far: f32, ) -> Mat4

👎Deprecated since 0.33.1:

use the glam::camera::rh::proj::directx::frustum function instead

Creates a right-handed perspective projection matrix with [0,1] depth range.

Panics

Will panic if left equals right, bottom equals top, z_near equals z_far, or z_near or z_far are not positive when glam_assert is enabled.

pub fn perspective_rh_gl( fov_y_radians: f32, aspect_ratio: f32, z_near: f32, z_far: f32, ) -> Mat4

👎Deprecated since 0.33.1:

use the glam::camera::rh::proj::opengl::perspective function instead

Creates a right-handed perspective projection matrix with [-1,1] depth range.

Useful to map the standard right-handed coordinate system into what OpenGL expects.

This is the same as the OpenGL gluPerspective function. See https://www.khronos.org/registry/OpenGL-Refpages/gl2.1/xhtml/gluPerspective.xml

pub fn perspective_lh( fov_y_radians: f32, aspect_ratio: f32, z_near: f32, z_far: f32, ) -> Mat4

👎Deprecated since 0.33.1:

use the glam::camera::lh::proj::directx::perspective function instead

Creates a left-handed perspective projection matrix with [0,1] depth range.

Useful to map the standard left-handed coordinate system into what WebGPU/Metal/Direct3D expect.

Panics

Will panic if fov_y_radians is not in the range (0, π), if aspect_ratio is zero, or if z_near or z_far are less than or equal to zero, or if z_near is equal to z_far, when glam_assert is enabled.

pub fn perspective_rh( fov_y_radians: f32, aspect_ratio: f32, z_near: f32, z_far: f32, ) -> Mat4

👎Deprecated since 0.33.1:

use the glam::camera::rh::proj::directx::perspective function instead

Creates a right-handed perspective projection matrix with [0,1] depth range.

Useful to map the standard right-handed coordinate system into what WebGPU/Metal/Direct3D expect.

Panics

Will panic if fov_y_radians is not in the range (0, π), if aspect_ratio is zero, or if z_near or z_far are less than or equal to zero, or if z_near is equal to z_far, when glam_assert is enabled.

pub fn perspective_infinite_lh( fov_y_radians: f32, aspect_ratio: f32, z_near: f32, ) -> Mat4

👎Deprecated since 0.33.1:

use the glam::camera::lh::proj::directx::perspective_infinite function instead

Creates an infinite left-handed perspective projection matrix with [0,1] depth range.

Like perspective_lh, but with an infinite value for z_far. The result is that points near z_near are mapped to depth 0, and as they move towards infinity the depth approaches 1.

Panics

Will panic if fov_y_radians is not in the range (0, π), if aspect_ratio is zero, or if z_near is less than or equal to zero when glam_assert is enabled.

pub fn perspective_infinite_reverse_lh( fov_y_radians: f32, aspect_ratio: f32, z_near: f32, ) -> Mat4

👎Deprecated since 0.33.1:

use the glam::camera::lh::proj::directx::perspective_infinite_reverse function instead

Creates an infinite reverse left-handed perspective projection matrix with [0,1] depth range.

Similar to perspective_infinite_lh, but maps Z = z_near to a depth of 1 and Z = infinity to a depth of 0.

Panics

Will panic if fov_y_radians is not in the range (0, π), if aspect_ratio is zero, or if z_near is less than or equal to zero when glam_assert is enabled.

pub fn perspective_infinite_rh( fov_y_radians: f32, aspect_ratio: f32, z_near: f32, ) -> Mat4

👎Deprecated since 0.33.1:

use the glam::camera::rh::proj::directx::perspective_infinite function instead

Creates an infinite right-handed perspective projection matrix with [0,1] depth range.

Like perspective_rh, but with an infinite value for z_far. The result is that points near z_near are mapped to depth 0, and as they move towards infinity the depth approaches 1.

Panics

Will panic if fov_y_radians is not in the range (0, π), if aspect_ratio is zero, or if z_near is less than or equal to zero when glam_assert is enabled.

pub fn perspective_infinite_reverse_rh( fov_y_radians: f32, aspect_ratio: f32, z_near: f32, ) -> Mat4

👎Deprecated since 0.33.1:

use the glam::camera::rh::proj::directx::perspective_infinite_reverse function instead

Creates an infinite reverse right-handed perspective projection matrix with [0,1] depth range.

Similar to perspective_infinite_rh, but maps Z = z_near to a depth of 1 and Z = infinity to a depth of 0.

Panics

Will panic if fov_y_radians is not in the range (0, π), if aspect_ratio is zero, or if z_near is less than or equal to zero when glam_assert is enabled.

pub fn orthographic_rh_gl( left: f32, right: f32, bottom: f32, top: f32, near: f32, far: f32, ) -> Mat4

👎Deprecated since 0.33.1:

use the glam::camera::rh::proj::opengl::orthographic function instead

Creates a right-handed orthographic projection matrix with [-1,1] depth range. This is the same as the OpenGL glOrtho function in OpenGL. See https://www.khronos.org/registry/OpenGL-Refpages/gl2.1/xhtml/glOrtho.xml

Useful to map a right-handed coordinate system to the normalized device coordinates that OpenGL expects.

pub fn orthographic_lh( left: f32, right: f32, bottom: f32, top: f32, near: f32, far: f32, ) -> Mat4

👎Deprecated since 0.33.1:

use the glam::camera::lh::proj::directx::orthographic function instead

Creates a left-handed orthographic projection matrix with [0,1] depth range.

Useful to map a left-handed coordinate system to the normalized device coordinates that WebGPU/Direct3D/Metal expect.

pub fn orthographic_rh( left: f32, right: f32, bottom: f32, top: f32, near: f32, far: f32, ) -> Mat4

👎Deprecated since 0.33.1:

use the glam::camera::rh::proj::directx::orthographic function instead

Creates a right-handed orthographic projection matrix with [0,1] depth range.

Useful to map a right-handed coordinate system to the normalized device coordinates that WebGPU/Direct3D/Metal expect.

pub fn project_point3(&self, rhs: Vec3) -> Vec3

Transforms the given 3D vector as a point, applying perspective correction.

This is the equivalent of multiplying the 3D vector as a 4D vector where w is 1.0. The perspective divide is performed meaning the resulting 3D vector is divided by w.

This method assumes that self contains a projective transform.

pub fn transform_point3(&self, rhs: Vec3) -> Vec3

Transforms the given 3D vector as a point.

This is the equivalent of multiplying the 3D vector as a 4D vector where w is 1.0.

This method assumes that self contains a valid affine transform. It does not perform a perspective divide, if self contains a perspective transform, or if you are unsure, the Self::project_point3() method should be used instead.

Panics

Will panic if the 3rd row of self is not (0, 0, 0, 1) when glam_assert is enabled.

pub fn transform_vector3(&self, rhs: Vec3) -> Vec3

Transforms the given 3D vector as a direction.

This is the equivalent of multiplying the 3D vector as a 4D vector where w is 0.0.

This method assumes that self contains a valid affine transform.

Panics

Will panic if the 3rd row of self is not (0, 0, 0, 1) when glam_assert is enabled.

pub fn project_point3a(&self, rhs: Vec3A) -> Vec3A

Transforms the given Vec3A as a 3D point, applying perspective correction.

This is the equivalent of multiplying the Vec3A as a 4D vector where w is 1.0. The perspective divide is performed meaning the resulting 3D vector is divided by w.

This method assumes that self contains a projective transform.

pub fn transform_point3a(&self, rhs: Vec3A) -> Vec3A

Transforms the given Vec3A as 3D point.

This is the equivalent of multiplying the Vec3A as a 4D vector where w is 1.0.

This method assumes that self contains a valid affine transform. If self contains a perspective transform, or if you are unsure, the Self::project_point3a() method should be used instead.

Panics

Will panic if the 3rd row of self is not (0, 0, 0, 1) when glam_assert is enabled.

pub fn transform_vector3a(&self, rhs: Vec3A) -> Vec3A

Transforms the given Vec3A as 3D vector.

This is the equivalent of multiplying the Vec3A as a 4D vector where w is 0.0.

This method assumes that self contains a valid affine transform.

Panics

Will panic if the 3rd row of self is not (0, 0, 0, 1) when glam_assert is enabled.

pub fn mul_vec4(&self, rhs: Vec4) -> Vec4

Transforms a 4D vector.

pub fn mul_transpose_vec4(&self, rhs: Vec4) -> Vec4

Transforms a 4D vector by the transpose of self.

pub fn mul_mat4(&self, rhs: &Mat4) -> Mat4

Multiplies two 4x4 matrices.

pub fn add_mat4(&self, rhs: &Mat4) -> Mat4

Adds two 4x4 matrices.

pub fn sub_mat4(&self, rhs: &Mat4) -> Mat4

Subtracts two 4x4 matrices.

pub fn mul_scalar(&self, rhs: f32) -> Mat4

Multiplies a 4x4 matrix by a scalar.

pub fn mul_diagonal_scale(&self, scale: Vec4) -> Mat4

Multiply self by a scaling vector scale. This is faster than creating a whole diagonal scaling matrix and then multiplying that. This operation is commutative.

pub fn div_scalar(&self, rhs: f32) -> Mat4

Divides a 4x4 matrix by a scalar.

pub fn recip(&self) -> Mat4

Returns a matrix containing the reciprocal 1.0/n of each element of self.

pub fn abs_diff_eq(&self, rhs: Mat4, max_abs_diff: f32) -> bool

Returns true if the absolute difference of all elements between self and rhs is less than or equal to max_abs_diff.

This can be used to compare if two matrices contain similar elements. It works best when comparing with a known value. The max_abs_diff that should be used used depends on the values being compared against.

For more see comparing floating point numbers.

pub fn abs(&self) -> Mat4

Takes the absolute value of each element in self

pub fn as_dmat4(&self) -> DMat4