pub trait ObserverSystemExt<E, B, M>: Sized + IntoObserverSystem<E, B, M>{
// Provided method
fn run_if<C, CM>(self, condition: C) -> ObserverWithCondition<E, B, M, Self>
where C: SystemCondition<CM> { ... }
}Expand description
Extension trait for adding run conditions to observer systems.
Provided Methods
Sourcefn run_if<C, CM>(self, condition: C) -> ObserverWithCondition<E, B, M, Self>where
C: SystemCondition<CM>,
fn run_if<C, CM>(self, condition: C) -> ObserverWithCondition<E, B, M, Self>where
C: SystemCondition<CM>,
Adds a run condition to this observer system.
The observer will only run if the condition returns true.
Multiple conditions can be chained (AND semantics).
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.